@@ -67,7 +67,7 @@ set_utf8 = function(x) {
 #' For \code{query} with a character \code{dsn} the query text is handed to
 #' 'ExecuteSQL' on the GDAL/OGR data set and will result in the creation of a
 #' new layer (and \code{layer} is ignored). See 'OGRSQL'
-#' \url{https://www.gdal.org/ogr_sql.html} for details. Please note that the
+#' \url{https://gdal.org/user/ogr_sql_dialect.html} for details. Please note that the
 #' 'FID' special field is driver-dependent, and may be either 0-based (e.g. ESRI
 #' Shapefile), 1-based (e.g. MapInfo) or arbitrary (e.g. OSM). Other features of
 #' OGRSQL are also likely to be driver dependent. The available layer names may
@@ -123,7 +123,7 @@ st_read.default = function(dsn, layer, ...) {
 }
 
 process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
-		stringsAsFactors = default.stringsAsFactors(), geometry_column = 1, as_tibble = FALSE) {
+		stringsAsFactors = ifelse(as_tibble, FALSE, default.stringsAsFactors()), geometry_column = 1, as_tibble = FALSE) {
 
 	which.geom = which(vapply(x, function(f) inherits(f, "sfc"), TRUE))
 
@@ -134,10 +134,13 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 	if (length(which.geom) == 0) {
 		warning("no simple feature geometries present: returning a data.frame or tbl_df",
 			call. = FALSE)
-		if (as_tibble)
-			return(tibble::as_tibble(x))
-		else
-			return(as.data.frame(x , stringsAsFactors = stringsAsFactors))
+		x = if (!as_tibble) {
+				if (any(sapply(x, is.list)))
+					warning("list-column(s) present: in case of failure, try read_sf or as_tibble=TRUE") # nocov
+				as.data.frame(x , stringsAsFactors = stringsAsFactors)
+			} else
+				tibble::as_tibble(x)
+		return(x)
 	}
 
 	nm = names(x)[which.geom]
@@ -148,16 +151,17 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 	list.cols = x[lc.other]
 	nm.lc = names(x)[lc.other]
 
-	x = if (length(x) == length(geom)) { # ONLY geometry column(s)
+	if (length(x) == length(geom)) { # ONLY geometry column(s)
 		if (as_tibble)
-			tibble::tibble(row.names = seq_along(geom[[1]]))[-1]
+			x <- tibble::tibble(row.names = seq_along(geom[[1]]))[-1]
 		else
-			data.frame(row.names = seq_along(geom[[1]]))
+			x <- data.frame(row.names = seq_along(geom[[1]]))
 	} else {
-		if (as_tibble)
-			tibble::as_tibble(set_utf8(x[-c(lc.other, which.geom)]))
-		else
-			as.data.frame(set_utf8(x[-c(lc.other, which.geom)]), stringsAsFactors = stringsAsFactors)
+		x <- as.data.frame(set_utf8(x[-c(lc.other, which.geom)]), stringsAsFactors = stringsAsFactors)
+		if (as_tibble) {
+			# "sf" class is added later by `st_as_sf` (and sets all the attributes)
+			x <- tibble::new_tibble(x, attributes(x), nrow = nrow(x))
+		}
 	}
 
 	for (i in seq_along(lc.other))
@@ -176,6 +180,7 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 }
 
 #' @name st_read
+#' @param fid_column_name character; name of column to write feature IDs to; defaults to not doing this
 #' @note The use of \code{system.file} in examples make sure that examples run regardless where R is installed:
 #' typical users will not use \code{system.file} but give the file name directly, either with full path or relative
 #' to the current working directory (see \link{getwd}). "Shapefiles" consist of several files with the same basename
@@ -183,7 +188,7 @@ process_cpl_read_ogr = function(x, quiet = FALSE, ..., check_ring_dir = FALSE,
 #' @export
 st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet = FALSE, geometry_column = 1L, type = 0,
 		promote_to_multi = TRUE, stringsAsFactors = default.stringsAsFactors(),
-		int64_as_string = FALSE, check_ring_dir = FALSE) {
+		int64_as_string = FALSE, check_ring_dir = FALSE, fid_column_name = character(0)) {
 
 	layer = if (missing(layer))
 		character(0)
@@ -196,7 +201,8 @@ st_read.character = function(dsn, layer, ..., query = NA, options = NULL, quiet
 	if (length(promote_to_multi) > 1)
 		stop("`promote_to_multi' should have length one, and applies to all geometry columns")
 
-	x = CPL_read_ogr(dsn, layer, query, as.character(options), quiet, type, promote_to_multi, int64_as_string)
+	x = CPL_read_ogr(dsn, layer, query, as.character(options), quiet, type, fid_column_name, 
+		promote_to_multi, int64_as_string)
 	process_cpl_read_ogr(x, quiet, check_ring_dir = check_ring_dir,
 		stringsAsFactors = stringsAsFactors, geometry_column = geometry_column, ...)
 }
@@ -295,18 +301,22 @@ abbreviate_shapefile_names = function(x) {
 #' to update (append to) the existing data source, e.g. adding a table to an existing database.
 #' @param delete_dsn logical; delete data source \code{dsn} before attempting to write?
 #' @param delete_layer logical; delete layer \code{layer} before attempting to write? (not yet implemented)
+#' @param fid_column_name character, name of column with feature IDs; if specified, this column is no longer written as feature attribute.
 #' @details columns (variables) of a class not supported are dropped with a warning. When deleting layers or
 #' data sources is not successful, no error is emitted. \code{delete_dsn} and \code{delete_layers} should be
 #' handled with care; the former may erase complete directories or databases.
 #' @seealso \link{st_drivers}
+#' @return \code{obj}, invisibly; in case \code{obj} is of class \code{sfc}, it is returned as an  \code{sf} object.
 #' @examples
 #' nc = st_read(system.file("shape/nc.shp", package="sf"))
-#' st_write(nc, "nc.shp")
-#' st_write(nc, "nc.shp", delete_layer = TRUE) # overwrites
+#' st_write(nc, paste0(tempdir(), "nc.shp"))
+#' st_write(nc, paste0(tempdir(), "nc.shp"), delete_layer = TRUE) # overwrites
 #' data(meuse, package = "sp") # loads data.frame from sp
 #' meuse_sf = st_as_sf(meuse, coords = c("x", "y"), crs = 28992)
-#' st_write(meuse_sf, "meuse.csv", layer_options = "GEOMETRY=AS_XY") # writes X and Y as columns
-#' st_write(meuse_sf, "meuse.csv", layer_options = "GEOMETRY=AS_WKT", delete_dsn=TRUE) # overwrites
+#' # writes X and Y as columns:
+#' st_write(meuse_sf, paste0(tempdir(), "meuse.csv"), layer_options = "GEOMETRY=AS_XY") 
+#' st_write(meuse_sf, paste0(tempdir(), "meuse.csv"), layer_options = "GEOMETRY=AS_WKT",
+#'   delete_dsn=TRUE) # overwrites
 #' \dontrun{
 #'  library(sp)
 #'  example(meuse, ask = FALSE, echo = FALSE)
@@ -332,7 +342,8 @@ st_write.sfc = function(obj, dsn, layer, ...) {
 st_write.sf = function(obj, dsn, layer = NULL, ...,
 		driver = guess_driver_can_write(dsn),
 		dataset_options = NULL, layer_options = NULL, quiet = FALSE, factorsAsCharacter = TRUE,
-		update = driver %in% db_drivers, delete_dsn = FALSE, delete_layer = FALSE) {
+		update = driver %in% db_drivers, delete_dsn = FALSE, delete_layer = FALSE, 
+		fid_column_name = NULL) {
 
 	if (missing(dsn))
 		stop("dsn should specify a data source or filename")
@@ -345,8 +356,9 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 		}
 		if (is.null(layer))
 			layer = deparse(substitute(obj))
-		return(dbWriteTable(dsn, name = layer, value = obj, ...,
-			factorsAsCharacter = factorsAsCharacter))
+		dbWriteTable(dsn, name = layer, value = obj, ...,
+			factorsAsCharacter = factorsAsCharacter)
+		return(invisible(obj))
 	} else if (!inherits(dsn, "character")) { # add methods for other dsn classes here...
 		stop(paste("no st_write method available for dsn of class", class(dsn)[1]))
 	}
@@ -380,23 +392,30 @@ st_write.sf = function(obj, dsn, layer = NULL, ...,
 		else
 			class(geom[[1]])[1]
 
+	fids = if (!is.null(fid_column_name)) {
+			fids = as.character(obj[[fid_column_name]])
+			obj[[fid_column_name]] = NULL
+			fids
+		} else
+			character(0)
+
 	ret = CPL_write_ogr(obj, dsn, layer, driver,
 		as.character(dataset_options), as.character(layer_options),
-		geom, dim, quiet, update, delete_dsn, delete_layer)
+		geom, dim, fids, quiet, update, delete_dsn, delete_layer)
 	if (ret == 1) { # try through temp file:
 		tmp = tempfile(fileext = paste0(".", tools::file_ext(dsn))) # nocov start
 		if (!quiet)
 			message(paste("writing first to temporary file", tmp))
 		if (CPL_write_ogr(obj, tmp, layer, driver,
 				as.character(dataset_options), as.character(layer_options),
-				geom, dim, quiet, update, delete_dsn, delete_layer) == 1)
+				geom, dim, fids, quiet, update, delete_dsn, delete_layer) == 1)
 			stop(paste("failed writing to temporary file", tmp))
 		if (!file.copy(tmp, dsn, overwrite = update || delete_dsn || delete_layer))
 			stop(paste("copying", tmp, "to", dsn, "failed"))
 		if (!file.remove(tmp))
 			warning(paste("removing", tmp, "failed"))
 	} # nocov end
-	invisible(NULL)
+	invisible(obj)
 }
 
 #' @name st_write
